import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

export interface ActionMappingAnnotationAttributes {
  value: string[];
}

export class ActionMappingAnnotation implements Annotation {
  public readonly attributes: ActionMappingAnnotationAttributes;

  public constructor(value: string | string[] | ActionMappingAnnotationAttributes) {

    if (undefined === value) {
      throw new Error(`value attribute must be specified.`);
    }

    if (TypeUtil.isString(value)) {
      this.attributes = {
        value: [<string>value],
      };
    } else if (TypeUtil.isArray(value)) {
      this.attributes = {
        value: <string[]>value,
      };
    } else {
      this.attributes = <ActionMappingAnnotationAttributes>value;
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('ActionMapping');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('ActionMapping');
  }

}

export const ActionMapping = Decorator.create(ActionMappingAnnotation);
